import os

from django.db import migrations, transaction
from django.db.utils import InternalError, OperationalError, ProgrammingError

with open(
    os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        '..',
        'sql',
        'event_triggers_for_tstzrange_constraint.sql',
    )
) as sql_file:
    sql_forwards = sql_file.read()


class Migration(migrations.Migration):

    dependencies = [
        ('agendas', '0127_remove_pricing_models'),
    ]

    operations = [
        migrations.RunSQL(sql=sql_forwards, reverse_sql=migrations.RunSQL.noop),
    ]

    def _check_db(self, project_state, schema_editor):
        try:
            with transaction.atomic():
                try:
                    # will fail if extension does not exist and can not be created
                    schema_editor.execute('CREATE EXTENSION IF NOT EXISTS btree_Gist SCHEMA public')
                except (OperationalError, ProgrammingError):
                    # if no extension, do not create trigger
                    return project_state
        except InternalError:
            return project_state

    def apply(self, project_state, schema_editor, *args, **kwargs):
        result = self._check_db(project_state, schema_editor)
        return result or super().apply(project_state, schema_editor, *args, **kwargs)

    def unapply(self, project_state, schema_editor, *args, **kwargs):
        result = self._check_db(project_state, schema_editor)
        return result or super().unapply(project_state, schema_editor, *args, **kwargs)
